from enum import StrEnum, auto


class TextColor(StrEnum):
    default = auto()
    gray = auto()
    brown = auto()
    orange = auto()
    yellow = auto()
    green = auto()
    blue = auto()
    purple = auto()
    pink = auto()
    red = auto()
    gray_background = auto()
    brown_background = auto()
    orange_background = auto()
    yellow_background = auto()
    green_background = auto()
    blue_background = auto()
    purple_background = auto()
    pink_background = auto()
    red_background = auto()


class CodeLanguage(StrEnum):
    abap = auto()
    arduino = auto()
    bash = auto()
    basic = auto()
    c = auto()
    clojure = auto()
    coffeescript = auto()
    cpp = "c++"
    csharp = "c#"
    css = auto()
    dart = auto()
    diff = auto()
    docker = auto()
    elixir = auto()
    elm = auto()
    erlang = auto()
    flow = auto()
    fortran = auto()
    fsharp = "f#"
    gherkin = auto()
    glsl = auto()
    go = auto()
    graphql = auto()
    groovy = auto()
    haskell = auto()
    html = auto()
    java = auto()
    javascript = auto()
    json = auto()
    julia = auto()
    kotlin = auto()
    latex = auto()
    less = auto()
    lisp = auto()
    livescript = auto()
    lua = auto()
    makefile = auto()
    markdown = auto()
    markup = auto()
    matlab = auto()
    mermaid = auto()
    nix = auto()
    objective_c = "objective-c"
    ocaml = auto()
    pascal = auto()
    perl = auto()
    php = auto()
    plain_text = "plain text"
    powershell = auto()
    prolog = auto()
    protobuf = auto()
    python = auto()
    r = auto()
    reason = auto()
    ruby = auto()
    rust = auto()
    sass = auto()
    scala = auto()
    scheme = auto()
    scss = auto()
    shell = auto()
    sql = auto()
    swift = auto()
    typescript = auto()
    vb_net = "vb.net"
    verilog = auto()
    vhdl = auto()
    visual_basic = "visual basic"
    webassembly = auto()
    xml = auto()
    yaml = auto()


class NumberFormat(StrEnum):
    number = auto()
    number_with_commas = auto()
    percent = auto()
    dollar = auto()
    canadian_dollar = auto()
    euro = auto()
    pound = auto()
    yen = auto()
    ruble = auto()
    rupee = auto()
    won = auto()
    yuan = auto()
    real = auto()
    lira = auto()
    rupiah = auto()
    franc = auto()
    hong_kong_dollar = auto()
    new_zealand_dollar = auto()
    krona = auto()
    norwegian_krone = auto()
    mexican_peso = auto()
    rand = auto()
    new_taiwan_dollar = auto()
    danish_krone = auto()
    zloty = auto()
    baht = auto()
    forint = auto()
    koruna = auto()
    shekel = auto()
    chilean_peso = auto()
    philippine_peso = auto()
    dirham = auto()
    colombian_peso = auto()
    riyal = auto()
    ringgit = auto()
    leu = auto()
    argentine_peso = auto()
    uruguayan_peso = auto()


class SelectOptionColor(StrEnum):
    default = auto()
    gray = auto()
    brown = auto()
    orange = auto()
    yellow = auto()
    green = auto()
    blue = auto()
    purple = auto()
    pink = auto()
    red = auto()
